@@ -48,7 +48,8 @@ async def fetch_remote_schema(
     if not verify_ssl:
         ssl_context.check_hostname = False
         ssl_context.verify_mode = ssl.CERT_NONE
-        logger.warning("SSL certificate verification disabled for schema fetch - only use in development!")
+        logger.warning(
+            "SSL certificate verification disabled for schema fetch - only use in development!")
 
     connector = aiohttp.TCPConnector(ssl=ssl_context)
     async with aiohttp.ClientSession(connector=connector) as session:
@@ -60,15 +61,18 @@ async def fetch_remote_schema(
         ) as response:
             if response.status != 200:
                 text = await response.text()
-                raise Exception(f"Failed to fetch schema from {url}: {response.status} - {text}")
+                raise Exception(
+                    f"Failed to fetch schema from {url}: {response.status} - {text}")
 
             result = await response.json()
 
             if "errors" in result:
-                raise Exception(f"GraphQL errors during introspection: {result['errors']}")
+                raise Exception(
+                    f"GraphQL errors during introspection: {result['errors']}")
 
             if "data" not in result:
-                raise Exception(f"No data in introspection response from {url}")
+                raise Exception(
+                    f"No data in introspection response from {url}")
 
             # Build the client schema from the introspection result
             schema = build_client_schema(result["data"])
@@ -112,7 +116,8 @@ def fetch_remote_schema_sync(
         # There's already a loop running, use nest_asyncio or create a task
         import concurrent.futures
         with concurrent.futures.ThreadPoolExecutor() as executor:
-            future = executor.submit(asyncio.run, fetch_remote_schema(url, headers, timeout, verify_ssl))
+            future = executor.submit(asyncio.run, fetch_remote_schema(
+                url, headers, timeout, verify_ssl))
             return future.result()
 
 
@@ -163,7 +168,9 @@ class RemoteGraphQLClient:
         if self.bearer_token:
             self.headers["Authorization"] = f"Bearer {self.bearer_token}"
 
-    def _clean_variables(self, variables: Optional[Dict[str, Any]], strategy: str = "remove") -> Optional[Dict[str, Any]]:
+    def _clean_variables(
+        self, variables: Optional[Dict[str, Any]], strategy: str = "remove"
+    ) -> Optional[Dict[str, Any]]:
         """
         Handle Undefined values in GraphQL variables with configurable strategies.
 
@@ -211,7 +218,8 @@ class RemoteGraphQLClient:
                             cleaned_list.append(cleaned_item)
                     elif isinstance(item, list):
                         # Recursively clean nested lists
-                        nested_result = self._clean_variables({"temp": item}, strategy)
+                        nested_result = self._clean_variables(
+                            {"temp": item}, strategy)
                         if nested_result and "temp" in nested_result:
                             cleaned_list.append(nested_result["temp"])
                         elif strategy == "null":
@@ -230,14 +238,17 @@ class RemoteGraphQLClient:
         Behavior (per tests):
         - If variables is None or empty, remove the entire variable declaration list from the header.
         - Otherwise, keep only declarations whose names exist in the variables dict.
-        - Do not touch parentheses elsewhere in the query body (e.g., field arguments).
+        Additionally, ensure the query body does not reference variables we removed from the header
+        by stripping corresponding "arg: $var" pairs to avoid server-side validation errors.
         """
         import re
 
         # Regex to match the operation header's variable declaration list
         # Handles: query MyOp(...), mutation MyOp(...), or anonymous: query (...)
-        op_pattern = re.compile(r"\b(query|mutation)\b\s*[A-Za-z_][A-Za-z0-9_]*?\s*\(([^)]*)\)", re.IGNORECASE)
-        anon_op_pattern = re.compile(r"\b(query|mutation)\b\s*\(([^)]*)\)", re.IGNORECASE)
+        op_pattern = re.compile(
+            r"\b(query|mutation)\b\s*[A-Za-z_][A-Za-z0-9_]*?\s*\(([^)]*)\)", re.IGNORECASE)
+        anon_op_pattern = re.compile(
+            r"\b(query|mutation)\b\s*\(([^)]*)\)", re.IGNORECASE)
 
         match = op_pattern.search(query) or anon_op_pattern.search(query)
         if not match:
@@ -248,6 +259,7 @@ class RemoteGraphQLClient:
 
         # If no variables provided, strip the entire declaration list
         if not variables:
+            # Remove header declarations entirely, do not touch the body
             return query.replace(full_block, full_block.split('(')[0].rstrip())
 
         kept: list[str] = []
@@ -263,11 +275,72 @@ class RemoteGraphQLClient:
 
         if kept:
             new_block = full_block.split('(')[0] + '(' + ', '.join(kept) + ')'
+            # Replace only the header, do not touch the body
             return query.replace(full_block, new_block, 1)
         else:
             # Remove parentheses entirely if nothing kept
             return query.replace(full_block, full_block.split('(')[0].rstrip(), 1)
 
+    def _remove_unused_variables_from_query_and_body(self, query: str, variables: Optional[Dict[str, Any]]) -> str:
+        """Strips unused variable declarations from the header AND removes corresponding usages in the body.
+
+        This is used only at execution time to prevent server-side errors like
+        "Variable '$x' is not defined" when optional variables are omitted.
+        Tests call _remove_unused_variables_from_query() and expect header-only behavior.
+        """
+        import re
+
+        # First perform header-level filtering using the tested function
+        new_query = self._remove_unused_variables_from_query(query, variables)
+
+        # If no variables provided, drop all usages in body
+        if not variables:
+            # Remove any ", arg: $var" or leading "(arg: $var, ...)" patterns
+            new_query = re.sub(
+                r"\s*,\s*\w+\s*:\s*\$[A-Za-z_][A-Za-z0-9_]*", "", new_query)
+            new_query = re.sub(
+                r"\(\s*\w+\s*:\s*\$[A-Za-z_][A-Za-z0-9_]*\s*,\s*", "(", new_query)
+            new_query = re.sub(r"\(\s*\)", "()", new_query)
+            new_query = re.sub(
+                r"\(\s*\w+\s*:\s*\$[A-Za-z_][A-Za-z0-9_]*\s*\)", "()", new_query)
+            return new_query
+
+        # Identify variables declared in header after filtering
+        header_match = re.search(
+            r"\b(query|mutation)\b\s*(?:[A-Za-z_][A-Za-z0-9_]*\s*)?\(([^)]*)\)", new_query, re.IGNORECASE)
+        kept_vars: set[str] = set()
+        if header_match:
+            for decl in header_match.group(2).split(','):
+                m = re.match(r"\s*\$([A-Za-z_][A-Za-z0-9_]*)", decl)
+                if m:
+                    kept_vars.add(m.group(1))
+
+        # Remove usages for variables NOT in kept_vars
+        # Determine original declared vars from the original query's header
+        original_match = re.search(
+            r"\b(query|mutation)\b\s*(?:[A-Za-z_][A-Za-z0-9_]*\s*)?\(([^)]*)\)", query, re.IGNORECASE)
+        original_vars: set[str] = set()
+        if original_match:
+            for decl in original_match.group(2).split(','):
+                m = re.match(r"\s*\$([A-Za-z_][A-Za-z0-9_]*)", decl)
+                if m:
+                    original_vars.add(m.group(1))
+
+        dropped = original_vars - kept_vars
+        for var_name in dropped:
+            # Remove usage patterns for this var
+            new_query = re.sub(
+                rf"\s*,\s*\w+\s*:\s*\${var_name}\b", "", new_query)
+            new_query = re.sub(
+                rf"\(\s*\w+\s*:\s*\${var_name}\s*,\s*", "(", new_query)
+            new_query = re.sub(
+                rf"\(\s*\w+\s*:\s*\${var_name}\s*\)", "()", new_query)
+            new_query = re.sub(rf"\b\w+\s*:\s*\${var_name}\b", "", new_query)
+            new_query = re.sub(r"\(\s*,\s*\)", "()", new_query)
+            new_query = re.sub(r",\s*,", ",", new_query)
+
+        return new_query
+
     def _transform_null_arrays(self, data: Any, parent_key: str = '', type_context: Optional[str] = None) -> Any:
         """Transform null values to empty arrays based on GraphQL schema types."""
         if isinstance(data, dict):
@@ -280,8 +353,10 @@ class RemoteGraphQLClient:
                     transformed[key] = []
                 else:
                     # For nested objects, get the type context from schema
-                    nested_type_context = self._get_field_type_context(key, current_type_context)
-                    transformed[key] = self._transform_null_arrays(value, key, nested_type_context)
+                    nested_type_context = self._get_field_type_context(
+                        key, current_type_context)
+                    transformed[key] = self._transform_null_arrays(
+                        value, key, nested_type_context)
             return transformed
         elif isinstance(data, list):
             # For lists, check if items should have array fields converted based on sibling analysis
@@ -297,7 +372,8 @@ class RemoteGraphQLClient:
                     for dict_item in dict_items:
                         for k, v in dict_item.items():
                             if k not in combined_siblings and isinstance(v, list):
-                                combined_siblings[k] = v  # Use the first non-null list we find
+                                # Use the first non-null list we find
+                                combined_siblings[k] = v
 
                     # Transform this item with enhanced sibling context
                     transformed_item = {}
@@ -305,11 +381,14 @@ class RemoteGraphQLClient:
                         if value is None and self._should_convert_to_array(key, value, combined_siblings, type_context):
                             transformed_item[key] = []
                         else:
-                            nested_type_context = self._get_field_type_context(key, type_context)
-                            transformed_item[key] = self._transform_null_arrays(value, key, nested_type_context)
+                            nested_type_context = self._get_field_type_context(
+                                key, type_context)
+                            transformed_item[key] = self._transform_null_arrays(
+                                value, key, nested_type_context)
                     transformed_items.append(transformed_item)
                 else:
-                    transformed_items.append(self._transform_null_arrays(item, parent_key, type_context))
+                    transformed_items.append(self._transform_null_arrays(
+                        item, parent_key, type_context))
 
             return transformed_items
         else:
@@ -358,15 +437,18 @@ class RemoteGraphQLClient:
             ) as response:
                 if response.status != 200:
                     text = await response.text()
-                    raise Exception(f"Failed to execute introspection query: {response.status} - {text}")
+                    raise Exception(
+                        f"Failed to execute introspection query: {response.status} - {text}")
 
                 result = await response.json()
                 if "errors" in result:
-                    raise Exception(f"GraphQL introspection errors: {result['errors']}")
+                    raise Exception(
+                        f"GraphQL introspection errors: {result['errors']}")
 
                 return result.get("data", {})
             # If we exit the retry loop without returning, raise a final error
-            raise Exception("Failed to execute query after multiple retry attempts.")
+            raise Exception(
+                "Failed to execute query after multiple retry attempts.")
         finally:
             if close_session:
                 await session.close()
@@ -425,7 +507,8 @@ class RemoteGraphQLClient:
                             self._array_fields_cache[type_name][field_name] = is_list
 
                             # Extract the actual return type name for this field
-                            return_type_name = self._extract_type_name(field_type)
+                            return_type_name = self._extract_type_name(
+                                field_type)
                             if return_type_name:
                                 # Create mapping from field name to its return type
                                 field_key = f"{type_name}.{field_name}"
@@ -436,7 +519,8 @@ class RemoteGraphQLClient:
                                     self._field_type_map[field_name] = return_type_name
 
             self._introspected = True
-            logger.debug(f"Schema introspected, found {len(self._array_fields_cache)} types")
+            logger.debug(
+                f"Schema introspected, found {len(self._array_fields_cache)} types")
 
         except Exception as e:
             logger.warning(f"Schema introspection failed: {e}")
@@ -476,7 +560,9 @@ class RemoteGraphQLClient:
 
         return None
 
-    def _should_convert_to_array(self, key: str, value: Any, siblings: Dict[str, Any], type_context: Optional[str] = None) -> bool:
+    def _should_convert_to_array(
+        self, key: str, value: Any, siblings: Dict[str, Any], type_context: Optional[str] = None
+    ) -> bool:
         """Determine if a null value should become an empty array based on GraphQL schema types."""
         if value is not None:
             return False
@@ -503,7 +589,8 @@ class RemoteGraphQLClient:
         if not self.verify_ssl:
             ssl_context.check_hostname = False
             ssl_context.verify_mode = ssl.CERT_NONE
-            logger.warning("SSL certificate verification disabled - only use in development!")
+            logger.warning(
+                "SSL certificate verification disabled - only use in development!")
         return ssl_context
 
     def _create_session(self) -> aiohttp.ClientSession:
@@ -610,11 +697,13 @@ class RemoteGraphQLClient:
         }
 
         # Clean variables using the configured strategy
-        cleaned_variables = self._clean_variables(variables, self.undefined_strategy)
+        cleaned_variables = self._clean_variables(
+            variables, self.undefined_strategy)
 
         # For "remove" strategy, also remove variable declarations from query
         if self.undefined_strategy == "remove":
-            cleaned_query = self._remove_unused_variables_from_query(query, cleaned_variables)
+            cleaned_query = self._remove_unused_variables_from_query_and_body(
+                query, cleaned_variables)
         else:
             # For "null" strategy, keep original query since variables are converted to null
             cleaned_query = query
@@ -680,18 +769,26 @@ DEBUG: GraphQL Request Processing:
                     text = await response.text()
                     # Friendly error messages for common statuses
                     if response.status == 504:
-                        raise Exception("Remote GraphQL endpoint timed out (HTTP 504). Please try again.")
+                        raise Exception(
+                            "Remote GraphQL endpoint timed out (HTTP 504). Please try again.")
                     if response.status in (502, 503):
-                        raise Exception(f"Remote GraphQL endpoint unavailable (HTTP {response.status}). Please try again.")
+                        raise Exception(
+                            f"Remote GraphQL endpoint unavailable (HTTP {response.status}). Please try again.")
 
-                    raise Exception(f"Failed to execute query: {response.status} - {text}")
+                    raise Exception(
+                        f"Failed to execute query: {response.status} - {text}")
 
                 result = await response.json()
 
                 if "errors" in result:
                     # Check for authentication-related errors in GraphQL response
                     error_messages = str(result['errors']).lower()
-                    if ('unauthorized' in error_messages or 'authentication' in error_messages or 'forbidden' in error_messages) and retry_on_auth_error:
+                    auth_errors = (
+                        'unauthorized' in error_messages or
+                        'authentication' in error_messages or
+                        'forbidden' in error_messages
+                    )
+                    if auth_errors and retry_on_auth_error:
                         if await self.refresh_token():
                             if close_session:
                                 await session.close()
@@ -708,11 +805,13 @@ DEBUG: GraphQL Request Processing:
 
                 # Transform null arrays to empty arrays to satisfy MCP output schema validation
                 data = result.get("data", {})
-                transformed_data = self._transform_null_arrays(data, type_context="Query")
+                transformed_data = self._transform_null_arrays(
+                    data, type_context="Query")
                 return transformed_data
 
         except aiohttp.ClientError:
-            raise Exception("Network error talking to remote GraphQL endpoint. Please try again.")
+            raise Exception(
+                "Network error talking to remote GraphQL endpoint. Please try again.")
         finally:
             if close_session:
                 await session.close()
