@@ -1,20 +1,17 @@
 import enum
-from graphql_api import GraphQLAPI
+from graphql_api import GraphQLAPI, field
 
 from graphql_mcp.server import GraphQLMCP
 
-api = GraphQLAPI()
-
 
 class PreferenceKey(str, enum.Enum):
     AI_MODEL = "ai_model"
     TOOLS_ENABLED = "tools_enabled"
 
 
-@api.type(is_root_type=True)
 class DemoApp:
 
-    @api.field
+    @field
     def set_preference_test(self, key: PreferenceKey, value: str) -> bool:
         """Set a preference"""
         if isinstance(key, PreferenceKey):
@@ -22,22 +19,24 @@ class DemoApp:
         else:
             return False
 
-    @api.field
+    @field
     def get_preference_test(self) -> dict:
         """Get a preference"""
         return {"key": "ai_model", "value": "x"}
 
 
-mcp_server = GraphQLMCP.from_api(api=api)
+mcp_server = GraphQLMCP.from_api(api=GraphQLAPI(root_type=DemoApp))
 
 
-# Add an addition tool
+# Add an addition standard fastmcp tool
 @mcp_server.tool()
 def clear_preferences() -> bool:
     """Clear all preferences"""
     return True
 
 
+app = mcp_server.http_app(transport="streamable-http", stateless_http=True)
+
 if __name__ == "__main__":
     import uvicorn
-    uvicorn.run(mcp_server.http_app, host="0.0.0.0", port=8010)
+    uvicorn.run(app, host="0.0.0.0", port=8010)
