@@ -3,7 +3,7 @@ import pytest
 
 from fastmcp.client import Client
 
-from graphql_mcp.server import GraphQLMCPServer
+from graphql_mcp.server import GraphQLMCP
 
 
 @pytest.mark.asyncio
@@ -26,7 +26,7 @@ async def test_graphql_generated_tool_schema_matches_direct_tool_schema():
             return True
 
     # Build the MCP server from the GraphQL API
-    mcp_server = GraphQLMCPServer.from_api(api, name="ParityTest")
+    mcp_server = GraphQLMCP.from_api(api, name="ParityTest")
 
     # Add a direct MCP tool with the same signature
     @mcp_server.tool
@@ -50,19 +50,35 @@ async def test_graphql_generated_tool_schema_matches_direct_tool_schema():
             defs = input_schema.get("$defs") or {}
             key_prop = (input_schema.get("properties") or {}).get("key") or {}
 
-            # Expect a $ref to a local enum def
-            assert "$ref" in key_prop, f"missing $ref in key property: {key_prop}"
-            ref = key_prop["$ref"]
-            assert ref.startswith("#/$defs/"), f"unexpected ref target: {ref}"
-            enum_name = ref.split("/#$defs/")[-1] if "/#$defs/" in ref else ref.split("#/$defs/")[-1]
-            assert enum_name in defs, f"enum def {enum_name} not found in $defs: {defs.keys()}"
-            enum_def = defs[enum_name]
-            assert enum_def.get("type") == "string", f"enum should be string-typed: {enum_def}"
-            assert enum_def.get("enum") == ["ai_model", "tools_enabled"], f"enum values mismatch: {enum_def}"
+            # Accept either $ref to enum def OR inline enum with expected values
+            if "$ref" in key_prop:
+                # $ref approach (direct MCP tools)
+                ref = key_prop["$ref"]
+                assert ref.startswith(
+                    "#/$defs/"), f"unexpected ref target: {ref}"
+                enum_name = ref.split(
+                    "/#$defs/")[-1] if "/#$defs/" in ref else ref.split("#/$defs/")[-1]
+                assert enum_name in defs, f"enum def {enum_name} not found in $defs: {defs.keys()}"
+                enum_def = defs[enum_name]
+                assert enum_def.get(
+                    "type") == "string", f"enum should be string-typed: {enum_def}"
+                assert enum_def.get("enum") == [
+                    "ai_model", "tools_enabled"], f"enum values mismatch: {enum_def}"
+            elif "enum" in key_prop:
+                # Inline enum approach (GraphQL tools with Literal types)
+                assert key_prop.get(
+                    "type") == "string", f"enum should be string-typed: {key_prop}"
+                enum_values = key_prop.get("enum", [])
+                # Should contain the enum values, and may also contain enum names
+                assert "ai_model" in enum_values, f"missing 'ai_model' in enum values: {enum_values}"
+                assert "tools_enabled" in enum_values, f"missing 'tools_enabled' in enum values: {enum_values}"
+            else:
+                assert False, f"key property should have either $ref or enum: {key_prop}"
 
             # Both args should be required
             required = input_schema.get("required") or []
-            assert set(required) == {"key", "value"}, f"required mismatch: {required}"
+            assert set(required) == {
+                "key", "value"}, f"required mismatch: {required}"
 
         assert_has_enum_schema(t_direct)
         assert_has_enum_schema(t_graphql)
@@ -76,7 +92,8 @@ async def test_graphql_generated_tool_schema_matches_direct_tool_schema():
             assert out.get("type") == "object"
             assert out.get("x-fastmcp-wrap-result") is True
             props = out.get("properties") or {}
-            assert "result" in props and props["result"].get("type") == "boolean"
+            assert "result" in props and props["result"].get(
+                "type") == "boolean"
 
         assert_has_wrapped_bool_output(t_direct)
         assert_has_wrapped_bool_output(t_graphql)
@@ -111,7 +128,8 @@ def test_graphql_api_enum_behavior():
             checkTag(tag: $tag)
         }
     """
-    result_enum = executor.execute(query_enum, variables={'tag': Tag.PYTHON.name})
+    result_enum = executor.execute(
+        query_enum, variables={'tag': Tag.PYTHON.name})
     assert result_enum.data == {'checkTag': True}
 
     # Schema should contain TagEnum
